<?php


/******************************
 * Matt Miller
 * Copyright 2009
 *
 * Short summary
 *  Database Abstraction Layer Utilizing PDO
 *
 * -----------------------------
 *
 * $Id: $
 * version: 0.9.5 04/09/09
 ******************************/

if (!defined("ACCESS"))die('BAD');
if (ACCESS != 'asdfasdf')die('BAD');
$gl['app_includes'][] = __FILE__;

class pdodb {
	private $db;
	public $sqlstrings; // Array of SQL strings executed or queried
	private $open_time; // Holder for when the db was connected
	private $close_time; // Holder for when the db was disconnected
	private $work_time; // Total CPU Time taken ( minus few flops for tracking )
	private $work_start_time; // Buffer to keep track of when work starts
	private $mySqli; // used for string escaping
	// Constructor
	public function pdodb($dsn = false, $un = '', $pw = '') {
		if ($dsn)
			return $this->connect($dsn,$un,$pw);
		else
			return;
	}

	private function start_work() {
		if ($this->work_start_time == 0.0)
			$this->work_start_time = floor(microtime(true) * 1000);
	}

	private function stop_work() {
		if ($this->work_start_time != 0.0) {
			$this->work_time += (floor(microtime(true) * 1000)) - $this->work_start_time;
			$this->work_start_time = 0.0;
		}
	}

	private function except($message) {
		$this->stop_work();
		//throw new Exception($message);
		trigger_error($message);
	}

	/**
	 * Connect to the database, and select it if $name is provided
	 *
	 * @param string $dsn database connection string
	 * @param string $db_prefix database prefix
	 * @return bool
	 */
	public function connect($dsn, $un = '', $pw = '') {
		try {
			$sqlstrings = null;
			$this->closeDb();
			if ($un != ''){
				$this->db = new PDO($dsn, $un, $pw);
			}
			else
			{
				$this->db = new PDO($dsn);
			}

			if ($this->db) {
				$this->open_time = floor(microtime(true) * 1000);
				$this->work_time = 0.0;				
				return true;
			} else {
				return false;
			}
		} catch (Exception $e) {
			$this->except($e);
		}
	}
 
	/**
	 * Get all queries/statements executed
	 *
	 * @return string[]
	 */
	public function getSQLStrings() {
		return $this->sqlstrings;
	}

	/**
	 * Get number of queries executed since the database was connected.
	 *
	 * @return int
	 */
	public function getNumQueries() {
		return count($this->sqlstrings);
	}

	/**
	 * Check last operation on the database handle or stmt for any errors
	 *
	 * @param PDOStatement $stmt
	 * @return void
	 */
	public function checkError($stmt = null) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			if (!$stmt) {
				if ($this->db->errorCode() != '00000') {
					$errorArray = $this->db->errorInfo();
					$errorArray[] = $this->sqlstrings;
					$this->except("<pre style='margin-bottom:0px;margin-top:0px;'>" . var_export($errorArray, true) . "</pre>");
				}
			} else {
				if ($stmt->errorCode() != '00000') {
					$errorArray = $stmt->errorInfo();
					$errorArray[] = $this->sqlstrings;
					$this->except("<pre style='margin-bottom:0px;margin-top:0px;'>" . var_export($errorArray, true) . "</pre>");
				}
			}
			$this->stop_work();
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Frees up the connection to the server so that other SQL stmts may be issued, 
	 * but leaves the stmt in a state that enables it to be executed again.
	 *
	 * @param string $sql
	 * @return bool
	 */
	public function free($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$bool = $stmt->closeCursor();
			$this->stop_work();
			return $bool;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Prepares a stmt for execution and returns a stmt object
	 *
	 * @param string $sql
	 * @return PDOStatement
	 */
	public function prepare($sql) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			$this->sqlstrings[] = $sql;
			$stmt = $this->db->prepare($sql);
			$this->stop_work();
			return $stmt;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Binds a PHP variable to a corresponding named or question mark placeholder in the SQL stmt that 
	 * was use to prepare the stmt. Unlike PDOstmt::bindValue(), the variable is bound as a 
	 * reference and will only be evaluated at the time that PDOstmt::execute() is called.
	 *
	 * @param PDOstmt $stmt
	 * @param mixed $parameter 	( Parameter identifier. For a prepared stmt using named placeholders, this will be a parameter name of the form :name. For a prepared stmt using question mark placeholders, this will be the 1-indexed position of the parameter. )
	 * @param mixed $var 		( Name of the PHP variable to bind to the SQL stmt parameter. )
	 * @param int $type 		( Explicit data type for the parameter using the PDO::PARAM_* constants. To return an INOUT parameter from a stored procedure, use the bitwise OR operator to set the PDO::PARAM_INPUT_OUTPUT bits for the data_type parameter. )
	 * @param int $length 		( Length of the data type. To indicate that a parameter is an OUT parameter from a stored procedure, you must explicitly set the length. )
	 * @param int $options 		( driver_options )
	 * @return void
	 */
	public function bindParam($stmt, $parameter, & $var, $type = false, $length = false, $options = false) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			if ($options && $length && $type) {
				$stmt->bindParam($parameter, $var, $type, $length, $options);
			}
			elseif (!$options && $length && $type) {
				$stmt->bindParam($parameter, $var, $type, $length);
			}
			elseif (!$options && !$length && $type) {
				$stmt->bindParam($parameter, $var, $type);
			}
			elseif (!$options && !$length && !$type) {
				$stmt->bindParam($parameter, $var);
			} else {
				$stmt->bindParam($parameter, $var);
			}
			$this->stop_work();
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * arranges to have a particular variable bound to a given column in the stmt-set from a query. 
	 * Each call to fetch_assoc, fetch_both, or fetch_obj will update all the variables that are bound to columns.
	 *
	 * @param PDOstmt $stmt
	 * @param mixed $column ( Number of the column (1-indexed) or name of the column in the stmt set. If using the column name, be aware that the name should match the case of the column, as returned by the driver. )
	 * @param mixed $var ( Name of the PHP variable to which the column will be bound. )
	 * @param int $type ( Data type of the parameter, specified by the PDO::PARAM_* constants. )
	 * @return void
	 */
	public function bindColumn($stmt, $col, & $var, $type = false) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			if (!$type) {
				$stmt->bindColumn($col, $var);
			} else {
				$stmt->bindColumn($col, $var, $type);
			}
			$this->stop_work();
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Binds a value to a corresponding named or question mark placeholder in the SQL stmt that was use to prepare the stmt.
	 *
	 * @param PDOstmt $stmt
	 * @param mixed $parameter ( Parameter identifier. For a prepared stmt using named placeholders, this will be a parameter name of the form :name. For a prepared stmt using question mark placeholders, this will be the 1-indexed position of the parameter. )
	 * @param mixed $val ( The value to bind to the parameter. )
	 * @param int $type ( Data type of the parameter, specified by the PDO::PARAM_* constants. )
	 * @return void
	 */
	public function bindValue($stmt, $parameter, $val, $type = false) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			if (!$type) {
				$stmt->bindValue($parameter, $val);
			} else {
				$stmt->bindValue($parameter, $val, $type);
			}
			$this->stop_work();
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Execute the prepared stmt.
	 *
	 * @param PDOstmt $stmt
	 * @param array $input ( An array of values with as many elements as there are bound parameters in the SQL stmt being executed. )
	 * @return bool
	 */
	public function execute($stmt, $input = false) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			if (!$input) {
				if(!$stmt){
					$this->except('<pre>'.'Statement is Invalid'.print_r(debug_backtrace(),true).'</pre>');
					return;
				}
				$success = $stmt->execute();
				$this->sqlstrings[] = "[ execute() ]";
				$this->checkError($stmt);
				$this->stop_work();
				return $success;
			} else {
				$success = $stmt->execute($input);
				$this->sqlstrings[] = "[ execute( <blockquote>" . var_export($input, true) . "</blockquote> ) ]";
				$this->checkError($stmt);
				$this->stop_work();
				return $success;
			}
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Initiates a transaction
	 *
	 * @return bool
	 */
	public function beginTransaction() {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			$bool = $this->db->beginTransaction();
			$this->stop_work();
			return $bool;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Rolls back a transaction
	 *
	 * @return bool
	 */
	public function rollBack() {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			$bool = $this->db->rollBack();
			$this->stop_work();
			return $bool;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Commits a transaction
	 *
	 * @return bool
	 */
	public function commit() {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			$this->stop_work();
			return $this->db->commit();
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Execute an SQL statement and return the number of affected rows
	 *
	 * @param string $sql
	 * @return int
	 */
	public function exec($sql) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			$count = $this->db->exec($sql);
			$this->sqlstrings[] = $sql;
			$this->checkError();
			$this->stop_work();
			return $count;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * retrieve some database information
	 *
	 * @return string[]
	 */
	public function getAttributes() {
		try {
			$this->start_work();
			$attributes = array (
				"AUTOCOMMIT",
				"ERRMODE",
				"CASE",
				"CLIENT_VERSION",
				"CONNECTION_STATUS",
				"ORACLE_NULLS",
				"PERSISTENT",
				"PREFETCH",
				"SERVER_INFO",
				"SERVER_VERSION",
				"TIMEOUT"
			);
			$attrArr = array ();
			foreach ($attributes as $val) {
				@ $attrArr[$val] = $this->db->getAttribute(constant("PDO::ATTR_$val"));
			}
			$this->stop_work();
			return $attrArr;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Set an attribute
	 *
	 * @param int $attribute
	 * @param mixed $value
	 * @return bool
	 */
	public function setAttribute($attribute, $value) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			$bool = $this->db->setAttribute($attribute, $value);
			$this->stop_work();
			return $bool;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * retrieve all available database drivers
	 *
	 * @return string[]
	 */
	public function getAvailableDrivers() {
		try {
			$this->start_work();
			$return = PDO :: getAvailableDrivers();
			$this->stop_work();
			return $return;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * for running simple queries
	 *
	 * @param string $sql
	 * @return mixed $stmt handle
	 */
	public function query($sql) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			$this->sqlstrings[] = $sql;
			if (stripos($sql, 'select') !== false) {
				$stmt = $this->db->query($sql);
				$this->checkError($stmt);
				$this->stop_work();
				return $stmt;
			} else {
				$count = $this->db->exec($sql);
				$this->checkError();
				$this->stop_work();
				return $count;
			}
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Returns the ID of the last inserted row or sequence value
	 *
	 * @param string $name
	 * @return string
	 */
	public function lastInsertId($name = false) {
		try {
			$this->start_work();
			if (!$this->db)
				throw new Exception('Database not connected.');
			if ($name) {
				$out = $this->db->lastInsertId($name);
				$this->stop_work();
				return $out;
			} else {
				$out = $this->db->lastInsertId();
				$this->stop_work();
				return $out;
			}
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Front-end for SQL num affected rows
	 *
	 * @param PDOStatement $stmt
	 * @return int $num_affected number of rows affected -1 for invalid stmt
	 */
	public function num_affected($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->rowCount();
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Front-end for SQL_fetch_assoc
	 *
	 * @param PDOStatement $stmt handle
	 * @returns an array indexed by column name as returned in your stmt set
	 */
	public function fetch_assoc($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->fetch(PDO :: FETCH_ASSOC);
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Front-end for SQL_fetch_array
	 *
	 * @param PDOStatement $stmt handle
	 * @returns an array indexed by both column name and 0-indexed column number as returned in your stmt set
	 */
	public function fetch_both($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->fetch(PDO :: FETCH_BOTH);
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Front-end for SQL_fetch_object
	 *
	 * @param PDOStatement $stmt handle
	 * @return object current db row
	 */
	public function fetch_obj($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->fetch(PDO :: FETCH_OBJ);
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * assign the values of the columns in the result set to the PHP variables to which they were bound with the PDOStatement::bindParam() or PDOStatement::bindColumn() methods.
	 *
	 * @param PDOStatement $stmt handle
	 * @return object current db row
	 */
	public function fetch_bound($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->fetch(PDO :: FETCH_BOUND);
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Returns an array containing all of the result set rows
	 *
	 * @param PDOStatement $stmt handle
	 * @param int $style
	 * @param int $column_index
	 * @return array
	 */
	public function fetchall($stmt, $style = PDO :: FETCH_BOTH) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			//$out = $stmt->fetchAll($style, $column_index);
			if ($style == PDO :: FETCH_BOTH)
				$out = $stmt->fetchAll();
			else
				$out = $stmt->fetchAll($style);
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
		return $out;
	}

	/**
	 * Returns a single column from the next row of a result set
	 *
	 * @param PDOStatement $stmt handle
	 * @param int $column_index ( 0-indexed number of the column you wish to retrieve from the row. If no value is supplied, PDOStatement::fetchColumn()  fetches the first column. )
	 * @return string
	 */
	public function fetchColumn($stmt, $column_index = 0) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->fetchColumn($column_index);
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Returns metadata for a column in a result set
	 *
	 * @param PDOStatement $stmt handle
	 * @param int $column_index ( The 0-indexed column in the result set. )
	 * @return array
	 */
	public function getColumnMeta($stmt, $column_index = 0) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->getColumnMeta($column_index);
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Advances to the next rowset in a multi-rowset statement handle
	 *
	 * @param PDOStatement $stmt handle
	 * @return bool
	 */
	public function nextRowset($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->nextRowset();
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Returns the number of columns in the stmt set
	 *
	 * @param int $stmt handle
	 * @return int number of columns in stmt
	 */
	public function columnCount($stmt) {
		try {
			$this->start_work();
			if (!$stmt) {
				throw new Exception('Invalid statement passed');
			}
			$out = $stmt->columnCount();
			$this->stop_work();
			return $out;
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Front-end for SQL_num_rows (VERY inefficient) also re-executes the statement 
	 *
	 * @param int $stmt handle
	 * @return int number of rows in stmt
	 */
	public function num_rows($stmt) {
		try {
			if ($stmt != null) {
				$this->start_work();
				$out = count($stmt->fetchAll());
				$stmt->closeCursor();
				$stmt->execute();
				$this->stop_work();
				return $out;
			} else {
				return -1;
			}
		} catch (Exception $e) {
			$this->except($e);
		}
	}


	
	function query_toarray($sql){
		$stmt = $this->query($sql);
		$out = array();
		if (!$stmt)return $out;
		//if (mysql_errno($gl['app_db']) != '') return $out;
		//if (@mysql_num_rows($stmt) == 0) return $out;
		while($row = $this->fetch_assoc($stmt)){
			$out[] = $row;
		}
		return $out;
	}

	function sanitize_sql($string){
		if(get_magic_quotes_gpc()){
			$string = stripslashes($string);
		}
		return mysql_real_escape_string($string, $this->mySqli);
	}

	/**
	 * Closes the DB connection
	 *
	 * @return bool
	 */
	public function closeDb() {
		try {
			$this->start_work();
			if ($this->db) {
				$this->db = null;
				$this->close_time = (time()) + microtime(true);
				$this->stop_work();
				return true;
			} else {
				$this->stop_work();
				return false;
			}
		} catch (Exception $e) {
			$this->except($e);
		}
	}

	/**
	 * Get total time between when the database was opened and when it was closed (or now), or the CPU time used to perform any work.
	 *
	 * @return STRING
	 */
	public function benchmark($total = false) {
		if ($total) {
			if ($this->db) {
				return number_format(((floor(microtime(true) * 1000)) - $this->open_time) / 1000, 3);
			} else {
				return number_format(($this->close_time - $this->open_time) / 1000, 3);
			}
		} else {
			return number_format($this->work_time / 1000, 3);
		}
	}
} //-END CLASS
?>